###################################################
### chunk number 1: Define problem function
###################################################
xint <- c(-1, 2)
yint <- c(-1, 2)
func <- function(x, y) x^3-3*x + y^3-3*y


###################################################
### chunk number 2: Create grid for evaluation
###################################################
library("lattice")   
g <- expand.grid(x = seq(xint[1], xint[2], 0.1), 
		y = seq(yint[1], yint[2], 0.1))
g$z <- func(g$x, g$y)


###################################################
### chunk number 3: Create-Figure
###################################################
library("lattice")   
g <- expand.grid(x = seq(xint[1], xint[2], 0.1), 
		y = seq(yint[1], yint[2], 0.1))
g$z <- func(g$x, g$y)
print( wireframe(z ~ x + y, data = g) )		


###################################################
### chunk number 4: Define function for serial integration
###################################################
integLoop <- function(func, xint, yint, n)
{
	erg <- 0
	xincr <- ( xint[2]-xint[1] ) / n
	yincr <- ( yint[2]-yint[1] ) / n
	for(xi in seq(xint[1], xint[2], length.out = n)){
		for(yi in seq(yint[1], yint[2], length.out = n)){
			box <- func(xi, yi) * xincr * yincr
			erg <- erg + box
		}
	}
	return(erg)
}


###################################################
### chunk number 5: Define function for vectorized integration
###################################################
integVec <- function(func, xint, yint, n)
{
	xincr <- ( xint[2]-xint[1] ) / n
	yincr <- ( yint[2]-yint[1] ) / n
	erg <- sum( 
			func( seq(xint[1], xint[2], length.out = n), 
				seq(yint[1], yint[2], length.out = n) ) 
			) * xincr * yincr * n 
	return(erg)
}


###################################################
### chunk number 6: Define function for apply-like integration
###################################################
integApply <- function (func, xint, yint, n)
{
	applyfunc <- function(xrange, xint, yint, n, func)
	{
		yrange <- seq(yint[1], yint[2], length.out = n)
		xincr <- ( xint[2]-xint[1] ) / n
		yincr <- ( yint[2]-yint[1] ) / n
		erg <- sum( sapply(xrange, function(x) 
					sum( func(x, yrange) 
				)) ) * xincr * yincr
		return(erg)
	}
	xrange <- seq(xint[1], xint[2], length.out = n)
	erg <- sapply(xrange, applyfunc, xint, yint, n, func)
	return( sum(erg) )
}


###################################################
### chunk number 7: Define slave function
###################################################
slavefunc<- function(id, nslaves, xint, yint, n, func){
    xrange <- seq(xint[1], xint[2], length.out = n)[seq(id, n, nslaves)]
    yrange <- seq(yint[1], yint[2], length.out = n)
    xincr <- ( xint[2]-xint[1] ) / n
    yincr <- ( yint[2]-yint[1] ) / n
    erg <- sapply(xrange, function(x) 
				sum( func(x, yrange ) )
			) * xincr * yincr
	return( sum(erg) )
}


###################################################
### chunk number 8: Define function to solve problem in parallel with Rmpi
###################################################
integRmpi <- function (func, xint, yint, n)
{
	nslaves <- mpi.comm.size()-1
	erg <- mpi.parSapply(1:nslaves, slavefunc, 
			nslaves, xint, yint, n, func)
	return( sum(erg) )
}


###################################################
### chunk number 9: Define function to solve problem in parallel with NWS
###################################################
integNWS <- function(sleigh, func, xint, yint, n)
{
	nslaves <- status(s)$numWorkers
	erg <- eachElem(sleigh, slavefunc, list(id = 1:nslaves), 
			list(nslaves = nslaves, xint=xint, 
				yint = yint, n = n, func = func))
	return( sum(unlist(erg)) )
}


###################################################
### chunk number 10: Define function to solve problem in parallel with snow
###################################################
integSnow <- function(cluster, func, xint, yint, n)
{
	nslaves <- length(cluster)
	erg <- clusterApplyLB(cluster, 1:nslaves, 
			slavefunc, nslaves, xint, yint, n, func)
	return( sum(unlist(erg)) )
}


###################################################
### chunk number 11: Define function to solve problem in parallel with snowfall
###################################################
integSnowFall <- function(func, xint, yint, n)
{
	nslaves <- sfCpus()
	erg <- sfClusterApplyLB(1:nslaves, 
			slavefunc, nslaves, xint, yint, n, func)
	return( sum(unlist(erg)) )
}


###################################################
### chunk number 12: 
###################################################
n <- 10000


###################################################
### chunk number 13: Calculate serial results eval=FALSE
###################################################
erg <- integLoop(func, xint, yint, n)
erg <- integVec(func, xint, yint, n)
erg <- integApply(func, xint, yint, n)


###################################################
### chunk number 14: Calculate parallel results with Rmpi eval=FALSE
###################################################
library("Rmpi")
mpi.spawn.Rslaves(nslaves = 10)
erg <- integRmpi(func, xint, yint, n)
mpi.close.Rslaves()


###################################################
### chunk number 15: Calculate parallel results with NWS eval=FALSE
###################################################
library("nws")
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
s <- sleigh(nodeList = mycluster[-1],  launch = sshcmd, workerCount = 10)
erg <- integNWS(s, func, xint, yint, n)
stopSleigh(s)


###################################################
### chunk number 16: Calculate parallel results with snow eval=FALSE
###################################################
library("snow")
c1 <- makeMPIcluster(10)
erg <- integSnow(c1, func, xint, yint, n)
stopCluster(c1)


###################################################
### chunk number 17: Calculate parallel results with snowfall eval=FALSE
###################################################
library("snowfall")
sfInit(parallel = TRUE, cpus = 10, type = "MPI")
erg <- integSnowFall(func, xint, yint, n)
sfStop()


###################################################
### chunk number 18: Measure computation time for serial code eval=FALSE
###################################################
rep <- 5
tLoop <- replicate(rep, system.time( integLoop(func, xint, yint, n) ))
summary(tLoop[3, ])
tVec <- replicate(rep, system.time( integVec(func, xint, yint, n) ))
summary(tVec[3, ])
tApply <- replicate(rep, system.time( integApply(func, xint, yint, n) ))
summary(tApply[3, ])


###################################################
### chunk number 19: Measure computation time for parallel code with Rmpi eval=FALSE
###################################################
library("Rmpi")
mpi.spawn.Rslaves(nslaves = 10)
tRmpi <- replicate(rep, system.time( 
		     integRmpi(func, xint, yint, n) 
))
summary(tRmpi[3, ])
mpi.close.Rslaves()


###################################################
### chunk number 20: Measure computation time for parallel code with NWS eval=FALSE
###################################################
library("nws")
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
s <- sleigh(nodeList = mycluster[-1],  launch = sshcmd, workerCount = 10)
tNWS <- replicate(rep, system.time( 
		     integNWS(s, func, xint, yint, n) 
))
summary(tNWS[3, ])
stopSleigh(s)


###################################################
### chunk number 21: Measure computation time for parallel code with snpw eval=FALSE
###################################################
library("snow")
c1 <- makeMPIcluster(10)
tSnow1 <- replicate(rep, system.time( 
		     integSnow(c1, func, xint, yint, n) 
))
summary(tSnow1[3])
stopCluster(c1)
c1 <- makePVMcluster(10)
tSnow2 <- replicate(rep, system.time( 
		     integSnow(c1, func, xint, yint, n) 
))
summary(tSnow2[3, ])
stopCluster(c1)
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
c1 <- makeNWScluster(mycluster[2:11])
tSnow3 <- replicate(rep, system.time( 
		     integSnow(c1, func, xint, yint, n) 
))
summary(tSnow3[3, ])
stopCluster(c1)
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
c1 <- makeSOCKcluster(mycluster[2:11])
tSnow4 <- replicate(rep, system.time( 
		     integSnow(c1, func, xint, yint, n) 
))
summary(tSnow4[3, ])
stopCluster(c1)


###################################################
### chunk number 22: Measure computation time for parallel code with snowfall eval=FALSE
###################################################
library("snowfall")
sfInit(parallel = TRUE, cpus = 10, type = "MPI")
tSnowFall1 <- replicate(rep, system.time( 
		     integSnowFall(func, xint, yint, n) 
))
summary(tSnowFall1[3, ])
sfStop()
sfInit(parallel = TRUE, cpus = 10, type = "PVM")
tSnowFall2 <- replicate(rep, system.time( 
		     integSnowFall(func, xint, yint, n) 
))
summary(tSnowFall2[3, ])
sfStop()
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt" ,what = "character")
sfInit(parallel = TRUE, cpus = 10, type = "NWS", 
	     socketHosts = mycluster[2:11])
tSnowFall3 <- replicate(rep, system.time( 
		     integSnowFall(func, xint, yint, n) 
))
summary(tSnowFall3[3, ])
sfStop()
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
sfInit(parallel = TRUE, cpus = 10, type = "SOCK", 
	     socketHosts = mycluster[2:11])
tSnowFall4 <- replicate(rep, system.time( 
		     integSnowFall(func, xint, yint, n) 
))
summary(tSnowFall4[3, ])
sfStop()


###################################################
### chunk number 23: Use snow package for time visualization eval=FALSE
###################################################
library("snow")
c1 <- makeMPIcluster(10)
visSnow1 <- snow.time( integSnow(c1, func, xint, yint, n) )
stopCluster(c1)
c1 <- makePVMcluster(10)
visSnow2 <- snow.time( integSnow(c1, func, xint, yint, n) )
stopCluster(c1)
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
c1 <- makeNWScluster(mycluster[2:11])
visSnow3 <- snow.time( integSnow(c1, func, xint, yint, n) )
stopCluster(c1)
system("cat $PBS_NODEFILE > cluster.txt")
mycluster <- scan(file = "cluster.txt", what = "character")
c1 <- makeSOCKcluster(mycluster[2:11])
visSnow4 <- snow.time( integSnow(c1, func, xint, yint, n) )
stopCluster(c1)


###################################################
### chunk number 24: Plot figure eval=FALSE
###################################################
pdf(file="snow_times_comp3.pdf")
dev.set()
par(mfcol=c(2, 2), oma=c(0, 0, 0, 0), mar=c(4, 4, 2.5, 1))
plot(visSnow1, title = "Cluster Usage - MPI")
plot(visSnow2, title = "Cluster Usage - PVM")
plot(visSnow3, title = "Cluster Usage - NWS")
plot(visSnow4, title = "Cluster Usage - Socket")
dev.off()
